package forum

import (
	"fmt"
	"net/http"
	"regexp"

	"code.nonshy.com/nonshy/website/pkg/config"
	"code.nonshy.com/nonshy/website/pkg/models"
	"code.nonshy.com/nonshy/website/pkg/session"
	"code.nonshy.com/nonshy/website/pkg/templates"
)

// Regular expressions
var (
	FragmentPattern = `[a-z0-9._-]{1,30}`
	FragmentRegexp  = regexp.MustCompile(
		fmt.Sprintf(`^(%s)$`, FragmentPattern),
	)

	// Forum path parameters.
	ForumPathRegexp = regexp.MustCompile(
		fmt.Sprintf(`^/f/(%s)`, FragmentPattern),
	)
	ForumPostRegexp = regexp.MustCompile(
		fmt.Sprintf(`^/f/(%s)/(post)`, FragmentPattern),
	)
	ForumThreadRegexp = regexp.MustCompile(
		fmt.Sprintf(`^/f/(%s)/(thread)/(\d+)`, FragmentPattern),
	)
)

// Landing page for forums.
func Landing() http.HandlerFunc {
	tmpl := templates.Must("forum/index.html")
	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		// Get the current user.
		currentUser, err := session.CurrentUser(r)
		if err != nil {
			session.FlashError(w, r, "Couldn't get current user: %s", err)
			templates.Redirect(w, "/")
			return
		}

		// Get all the categorized index forums.
		// XXX: we get a large page size to get ALL official forums
		var pager = &models.Pagination{
			Page:    1,
			PerPage: config.PageSizeForums,
			Sort:    "title asc",
		}
		pager.ParsePage(r)

		forums, err := models.PaginateForums(currentUser, config.ForumCategories, pager)
		if err != nil {
			session.FlashError(w, r, "Couldn't paginate forums: %s", err)
			templates.Redirect(w, "/")
			return
		}

		// Bucket the forums into their categories for easy front-end.
		categorized := models.CategorizeForums(forums, config.ForumCategories)

		// Map statistics for these forums.
		forumMap := models.MapForumStatistics(forums)

		var vars = map[string]interface{}{
			"Pager":      pager,
			"Categories": categorized,
			"ForumMap":   forumMap,
		}
		if err := tmpl.Execute(w, r, vars); err != nil {
			http.Error(w, err.Error(), http.StatusInternalServerError)
			return
		}
	})
}
